/*
 * Copyright (c) 2007, 2009, Oracle and/or its affiliates. All rights reserved.
 * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/*
 * $Id: GenericTCKServlet.java 62571 2011-04-18 20:58:13Z kgrucci $
 */

/*
 * $URL: https://adc4110351.us.oracle.com/svn/spider/branches/cts/8.0/src/com/sun/ts/tests/servlet/common/servlets/GenericTCKServlet.java $ $LastChangedDate: 2011-04-18 13:58:13 -0700 (Mon, 18 Apr 2011) $
 */


package com.sun.ts.tests.servlet.common.servlets;

import javax.servlet.GenericServlet;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

/**
 * GenericTCKServlet.java
 *
 * Any test that would normally extend GenericServlet will
 * instead extend this class.  This will provide a simple
 * framework from invoking various tests defined as methods
 * within the servlet that extends this class.
 *
 * Created: Wed Jul 31 20:57:16 2002
 *
 * @author <a href="mailto:">Ryan Lubke</a>
 * @version %I%
 */

public abstract class GenericTCKServlet extends GenericServlet {

    /**
     * <code>TEST_HEADER</code> is the constant for the
     * <code>testname</code> header.
     */
    private static final String TEST_HEADER = "testname";

    /**
     * <code>TEST_ARGS</code> is an array of Classes used
     * during reflection.
     */
    private static final Class[] TEST_ARGS = {ServletRequest.class,
                                              ServletResponse.class};

    /**
     * <code>init</code> initializes the servlet.
     *
     * @param config - <code>ServletConfig</code>
     */
    public void init(ServletConfig config) throws ServletException {
        super.init(config);
    }

    public void service(ServletRequest req, ServletResponse res)
        throws ServletException, IOException {
        invokeTest(req, res);
    }

    /**
     * <code>invokeTest</code> uses reflection to invoke test methods
     * in child classes of this particular class.
     *
     * @param req - <code>ServletRequest</code>
     * @param res - <code>ServletResponse</code>
     * @exception ServletException if an error occurs
     */
    protected void invokeTest(ServletRequest req, ServletResponse res)
        throws ServletException {
        String test = req.getParameter(TEST_HEADER);
        try {
            Method method = this.getClass().getMethod(test, TEST_ARGS);
            method.invoke(this, new Object[]{req, res});
        } catch (InvocationTargetException ite) {
            throw new ServletException(ite.getTargetException());
        } catch (NoSuchMethodException nsme) {
            throw new ServletException("Test: " + test + " does not exist");
        } catch (Throwable t) {
            throw new ServletException("Error executing test: " + test, t);
        }
    }

}// GenericTCKServlet
